from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('djinsight', '0003_alter_pageviewlog_session_key'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentTypeRegistry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enabled', models.BooleanField(default=True, verbose_name='Enabled')),
                ('track_anonymous', models.BooleanField(default=True, verbose_name='Track Anonymous')),
                ('track_authenticated', models.BooleanField(default=True, verbose_name='Track Authenticated')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', unique=True, verbose_name='Content Type')),
            ],
            options={
                'verbose_name': 'Content Type Registry',
                'verbose_name_plural': 'Content Type Registries',
            },
        ),
        migrations.CreateModel(
            name='PageViewStatistics',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField(db_index=True)),
                ('total_views', models.PositiveIntegerField(default=0, verbose_name='Total Views')),
                ('unique_views', models.PositiveIntegerField(default=0, verbose_name='Unique Views')),
                ('first_viewed_at', models.DateTimeField(blank=True, null=True, verbose_name='First Viewed At')),
                ('last_viewed_at', models.DateTimeField(blank=True, null=True, verbose_name='Last Viewed At')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated At')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': 'Page View Statistics',
                'verbose_name_plural': 'Page View Statistics',
                'ordering': ['-total_views'],
            },
        ),
        migrations.CreateModel(
            name='PageViewEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('url', models.CharField(max_length=500, verbose_name='URL')),
                ('session_key', models.CharField(db_index=True, max_length=255, verbose_name='Session Key')),
                ('ip_address', models.GenericIPAddressField(blank=True, null=True, verbose_name='IP Address')),
                ('user_agent', models.TextField(blank=True, null=True, verbose_name='User Agent')),
                ('referrer', models.URLField(blank=True, max_length=500, null=True, verbose_name='Referrer')),
                ('timestamp', models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Timestamp')),
                ('is_unique', models.BooleanField(default=False, verbose_name='Is Unique')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': 'Page View Event',
                'verbose_name_plural': 'Page View Events',
                'ordering': ['-timestamp'],
            },
        ),
        migrations.AlterField(
            model_name='pageviewsummary',
            name='content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='pageviewsummary',
            name='page_id',
            field=models.PositiveIntegerField(db_index=True, verbose_name='Object ID'),
        ),
        migrations.AddIndex(
            model_name='contenttyperegistry',
            index=models.Index(fields=['content_type', 'enabled'], name='djinsight_c_content_idx'),
        ),
        migrations.AddIndex(
            model_name='pageviewstatistics',
            index=models.Index(fields=['content_type', 'object_id'], name='djinsight_p_content_obj_idx'),
        ),
        migrations.AddIndex(
            model_name='pageviewstatistics',
            index=models.Index(fields=['content_type', 'total_views'], name='djinsight_p_content_tot_idx'),
        ),
        migrations.AddIndex(
            model_name='pageviewstatistics',
            index=models.Index(fields=['content_type', 'unique_views'], name='djinsight_p_content_uni_idx'),
        ),
        migrations.AddIndex(
            model_name='pageviewstatistics',
            index=models.Index(fields=['last_viewed_at'], name='djinsight_p_last_vi_idx'),
        ),
        migrations.AddIndex(
            model_name='pageviewstatistics',
            index=models.Index(fields=['updated_at'], name='djinsight_p_updated_idx'),
        ),
        migrations.AlterUniqueTogether(
            name='pageviewstatistics',
            unique_together={('content_type', 'object_id')},
        ),
        migrations.AddIndex(
            model_name='pageviewevent',
            index=models.Index(fields=['content_type', 'object_id', 'timestamp'], name='djinsight_p_content_obj_tim_idx'),
        ),
        migrations.AddIndex(
            model_name='pageviewevent',
            index=models.Index(fields=['session_key', 'content_type', 'object_id'], name='djinsight_p_session_con_idx'),
        ),
        migrations.AddIndex(
            model_name='pageviewevent',
            index=models.Index(fields=['timestamp'], name='djinsight_p_timesta_idx'),
        ),
        migrations.AddIndex(
            model_name='pageviewevent',
            index=models.Index(fields=['content_type', 'timestamp'], name='djinsight_p_content_tim_idx'),
        ),
        migrations.RenameField(
            model_name='pageviewsummary',
            old_name='page_id',
            new_name='object_id',
        ),
        migrations.AlterField(
            model_name='pageviewsummary',
            name='content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddIndex(
            model_name='pageviewsummary',
            index=models.Index(fields=['content_type', 'object_id', 'date'], name='djinsight_p_content_obj_dat_idx'),
        ),
        migrations.AlterUniqueTogether(
            name='pageviewsummary',
            unique_together={('content_type', 'object_id', 'date')},
        ),
    ]



from django.contrib import admin
from django.utils.html import format_html

from djinsight.models import (
    ContentTypeRegistry,
    MCPAPIKey,
    PageViewEvent,
    PageViewStatistics,
    PageViewSummary,
)


@admin.register(ContentTypeRegistry)
class ContentTypeRegistryAdmin(admin.ModelAdmin):
    list_display = [
        "content_type",
        "enabled",
        "track_anonymous",
        "track_authenticated",
        "created_at",
    ]
    list_filter = ["enabled", "track_anonymous", "track_authenticated"]
    search_fields = ["content_type__app_label", "content_type__model"]
    readonly_fields = ["created_at", "updated_at"]


@admin.register(PageViewStatistics)
class PageViewStatisticsAdmin(admin.ModelAdmin):
    list_display = [
        "content_type",
        "object_id",
        "total_views",
        "unique_views",
        "view_ratio",
        "last_viewed_at",
    ]
    list_filter = ["content_type", "last_viewed_at"]
    search_fields = ["object_id"]
    readonly_fields = [
        "content_type",
        "object_id",
        "total_views",
        "unique_views",
        "first_viewed_at",
        "last_viewed_at",
        "updated_at",
    ]
    ordering = ["-total_views"]

    @admin.display(description="Unique Ratio")
    def view_ratio(self, obj):
        if obj.total_views > 0:
            ratio = (obj.unique_views / obj.total_views) * 100
            color = "green" if ratio > 50 else "orange" if ratio > 25 else "red"
            return format_html(
                '<span style="color: {};">{}%</span>', color, f"{ratio:.1f}"
            )
        return "-"

    def has_add_permission(self, request):
        return False


@admin.register(PageViewEvent)
class PageViewEventAdmin(admin.ModelAdmin):
    list_display = [
        "content_type",
        "object_id",
        "session_key_short",
        "timestamp",
        "is_unique",
    ]
    list_filter = ["content_type", "is_unique", "timestamp"]
    search_fields = ["object_id", "session_key", "url"]
    readonly_fields = [
        "content_type",
        "object_id",
        "url",
        "session_key",
        "ip_address",
        "user_agent",
        "referrer",
        "timestamp",
        "is_unique",
    ]
    date_hierarchy = "timestamp"
    ordering = ["-timestamp"]

    def session_key_short(self, obj):
        return f"{obj.session_key[:8]}..." if obj.session_key else "-"

    session_key_short.short_description = "Session"

    def has_add_permission(self, request):
        return False


@admin.register(PageViewSummary)
class PageViewSummaryAdmin(admin.ModelAdmin):
    list_display = [
        "content_type",
        "object_id",
        "date",
        "total_views",
        "unique_views",
        "view_ratio",
    ]
    list_filter = ["content_type", "date"]
    search_fields = ["object_id"]
    readonly_fields = [
        "content_type",
        "object_id",
        "date",
        "total_views",
        "unique_views",
    ]
    date_hierarchy = "date"
    ordering = ["-date"]

    @admin.display(description="Unique Ratio")
    def view_ratio(self, obj):
        if obj.total_views > 0:
            ratio = (obj.unique_views / obj.total_views) * 100
            color = "green" if ratio > 50 else "orange" if ratio > 25 else "red"
            return format_html(
                '<span style="color: {};">{}%</span>', color, f"{ratio:.1f}"
            )
        return "-"

    def has_add_permission(self, request):
        return False


@admin.register(MCPAPIKey)
class MCPAPIKeyAdmin(admin.ModelAdmin):
    list_display = ["name", "key_masked", "is_active", "created_at", "last_used_at"]
    list_filter = ["is_active", "created_at", "last_used_at"]
    search_fields = ["name", "description"]
    readonly_fields = ["key_display", "created_at", "last_used_at"]
    fields = [
        "name",
        "description",
        "is_active",
        "key_display",
        "created_at",
        "last_used_at",
    ]

    def key_masked(self, obj):
        if len(obj.key) > 12:
            return f"{obj.key[:8]}...{obj.key[-4:]}"
        return obj.key[:8] + "..."

    key_masked.short_description = "API Key"

    def key_display(self, obj):
        if obj.pk:
            return format_html(
                '<code style="background: #f5f5f5; padding: 8px; display: inline-block; font-size: 12px;">{}</code>',
                obj.key,
            )
        return "Will be generated on save"

    key_display.short_description = "API Key (Full)"

    def save_model(self, request, obj, form, change):
        if not change:
            obj.key = MCPAPIKey.generate_key()
        super().save_model(request, obj, form, change)



from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand
from django.db import transaction

from djinsight.models import (
    ContentTypeRegistry,
    PageViewEvent,
    PageViewStatistics,
    PageViewSummary,
)


class Command(BaseCommand):
    help = 'Migrate data from djinsight v0.1.x to v0.2.0'

    def add_arguments(self, parser):
        parser.add_argument(
            '--dry-run',
            action='store_true',
            help='Run migration without saving changes',
        )
        parser.add_argument(
            '--batch-size',
            type=int,
            default=1000,
            help='Batch size for processing records',
        )

    def handle(self, *args, **options):
        dry_run = options['dry_run']
        batch_size = options['batch_size']

        if dry_run:
            self.stdout.write(self.style.WARNING('DRY RUN MODE - No changes will be saved'))

        self.stdout.write('Starting migration from v0.1.x to v0.2.0...')

        try:
            migrated_logs = self._migrate_page_view_logs(dry_run, batch_size)
            migrated_summaries = self._migrate_summaries(dry_run, batch_size)
            migrated_stats = self._migrate_statistics(dry_run, batch_size)
            registered_types = self._register_content_types(dry_run)

            self.stdout.write(self.style.SUCCESS(f'\nMigration completed successfully!'))
            self.stdout.write(f'- Page View Events migrated: {migrated_logs}')
            self.stdout.write(f'- Summaries migrated: {migrated_summaries}')
            self.stdout.write(f'- Statistics migrated: {migrated_stats}')
            self.stdout.write(f'- Content Types registered: {registered_types}')

            if not dry_run:
                self.stdout.write(
                    self.style.WARNING('\nIMPORTANT: Old tables (PageViewLog with mixin fields) can now be safely removed.')
                )

        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Migration failed: {e}'))
            raise

    def _migrate_page_view_logs(self, dry_run, batch_size):
        try:
            from djinsight.models import PageViewLog as OldPageViewLog

            count = 0
            queryset = OldPageViewLog.objects.all().order_by('id')
            total = queryset.count()

            if total == 0:
                return 0

            self.stdout.write(f'\nMigrating {total} PageViewLog records...')

            for i in range(0, total, batch_size):
                batch = list(queryset[i:i + batch_size])
                new_events = []

                for log in batch:
                    try:
                        parts = log.content_type.split('.')
                        if len(parts) != 2:
                            continue

                        app_label, model = parts
                        content_type = ContentType.objects.get(app_label=app_label, model=model.lower())

                        new_events.append(PageViewEvent(
                            content_type=content_type,
                            object_id=log.page_id,
                            url=log.url,
                            session_key=log.session_key or '',
                            ip_address=log.ip_address,
                            user_agent=log.user_agent,
                            referrer=log.referrer,
                            timestamp=log.timestamp,
                            is_unique=log.is_unique,
                        ))
                    except Exception as e:
                        self.stdout.write(self.style.WARNING(f'Skipping log {log.id}: {e}'))
                        continue

                if not dry_run and new_events:
                    PageViewEvent.objects.bulk_create(new_events, batch_size=500, ignore_conflicts=True)

                count += len(new_events)
                if i % (batch_size * 5) == 0:
                    self.stdout.write(f'Processed {count}/{total} logs...')

            return count
        except ImportError:
            self.stdout.write(self.style.WARNING('Old PageViewLog model not found, skipping...'))
            return 0

    def _migrate_summaries(self, dry_run, batch_size):
        try:
            from djinsight.models import PageViewSummary as OldPageViewSummary

            count = 0
            old_queryset = OldPageViewSummary.objects.all().order_by('id')
            total = old_queryset.count()

            if total == 0:
                return 0

            self.stdout.write(f'\nMigrating {total} PageViewSummary records...')

            for i in range(0, total, batch_size):
                batch = list(old_queryset[i:i + batch_size])
                new_summaries = []

                for summary in batch:
                    try:
                        parts = summary.content_type.split('.')
                        if len(parts) != 2:
                            continue

                        app_label, model = parts
                        content_type_obj = ContentType.objects.get(app_label=app_label, model=model.lower())

                        existing = PageViewSummary.objects.filter(
                            content_type=content_type_obj,
                            object_id=summary.page_id,
                            date=summary.date
                        ).exists()

                        if not existing:
                            new_summaries.append(PageViewSummary(
                                content_type=content_type_obj,
                                object_id=summary.page_id,
                                date=summary.date,
                                total_views=summary.total_views,
                                unique_views=summary.unique_views,
                            ))
                    except Exception as e:
                        self.stdout.write(self.style.WARNING(f'Skipping summary {summary.id}: {e}'))
                        continue

                if not dry_run and new_summaries:
                    PageViewSummary.objects.bulk_create(new_summaries, batch_size=500, ignore_conflicts=True)

                count += len(new_summaries)
                if i % (batch_size * 5) == 0:
                    self.stdout.write(f'Processed {count}/{total} summaries...')

            return count
        except ImportError:
            return 0

    def _migrate_statistics(self, dry_run, batch_size):
        from django.apps import apps

        count = 0
        self.stdout.write('\nMigrating statistics from models with PageViewStatisticsMixin...')

        for model in apps.get_models():
            if not all(hasattr(model, f) for f in ['total_views', 'unique_views', 'first_viewed_at', 'last_viewed_at']):
                continue

            content_type = ContentType.objects.get_for_model(model)
            queryset = model.objects.all()
            total = queryset.count()

            if total == 0:
                continue

            self.stdout.write(f'Found {total} objects in {content_type}...')

            for i in range(0, total, batch_size):
                batch = list(queryset[i:i + batch_size])
                new_stats = []

                for obj in batch:
                    if obj.total_views == 0 and obj.unique_views == 0:
                        continue

                    existing = PageViewStatistics.objects.filter(
                        content_type=content_type,
                        object_id=obj.pk
                    ).exists()

                    if not existing:
                        new_stats.append(PageViewStatistics(
                            content_type=content_type,
                            object_id=obj.pk,
                            total_views=obj.total_views,
                            unique_views=obj.unique_views,
                            first_viewed_at=obj.first_viewed_at,
                            last_viewed_at=obj.last_viewed_at,
                        ))

                if not dry_run and new_stats:
                    PageViewStatistics.objects.bulk_create(new_stats, batch_size=500, ignore_conflicts=True)

                count += len(new_stats)

            self.stdout.write(f'Migrated {count} statistics from {content_type}')

        return count

    def _register_content_types(self, dry_run):
        from django.apps import apps

        count = 0
        self.stdout.write('\nRegistering content types...')

        tracked_models = set()

        for model in apps.get_models():
            if all(hasattr(model, f) for f in ['total_views', 'unique_views', 'first_viewed_at', 'last_viewed_at']):
                tracked_models.add(model)

        for event in PageViewEvent.objects.values('content_type').distinct():
            try:
                content_type = ContentType.objects.get(id=event['content_type'])
                model_class = content_type.model_class()
                if model_class:
                    tracked_models.add(model_class)
            except:
                pass

        for model in tracked_models:
            content_type = ContentType.objects.get_for_model(model)

            if not dry_run:
                ContentTypeRegistry.objects.get_or_create(
                    content_type=content_type,
                    defaults={'enabled': True}
                )
            count += 1
            self.stdout.write(f'Registered: {content_type}')

        return count



from django.core.management.base import BaseCommand, CommandError

from djinsight.tasks import run_process_page_views


class Command(BaseCommand):
    help = "Process page views from Redis and store them in the database"

    def add_arguments(self, parser):
        parser.add_argument(
            "--batch-size",
            type=int,
            default=1000,
            help="Number of records to process in a single transaction (default: 1000)",
        )
        parser.add_argument(
            "--max-records",
            type=int,
            default=10000,
            help="Maximum number of records to process in a single run (default: 10000)",
        )

    def handle(self, *args, **options):
        batch_size = options["batch_size"]
        max_records = options["max_records"]
        verbosity = options["verbosity"]

        if verbosity >= 1:
            self.stdout.write(
                self.style.SUCCESS(
                    f"Starting to process page views with batch_size={batch_size}, max_records={max_records}"
                )
            )

        try:
            processed = run_process_page_views(
                verbosity=verbosity, batch_size=batch_size, max_records=max_records
            )

            if verbosity >= 1:
                self.stdout.write(
                    self.style.SUCCESS(f"Successfully processed {processed} page views")
                )

        except Exception as e:
            raise CommandError(f"Error processing page views: {e}")



"""Tests for djinsight views."""

import json

from django.contrib.contenttypes.models import ContentType
from django.test import Client, TestCase
from django.urls import reverse

from djinsight.models import ContentTypeRegistry, MCPAPIKey, PageViewStatistics


class RecordPageViewTest(TestCase):
    """Test record_page_view endpoint."""

    def setUp(self):
        """Set up test data."""
        self.client = Client()
        self.content_type = ContentType.objects.get_for_model(PageViewStatistics)
        ContentTypeRegistry.register(PageViewStatistics)

    def test_record_view_creates_event(self):
        """Test that posting to record_page_view creates an event."""
        data = {
            "content_type": f"{self.content_type.app_label}.{self.content_type.model}",
            "object_id": 1,
            "url": "/test/",
            "referrer": "https://example.com",
            "user_agent": "Test Agent",
        }

        response = self.client.post(
            reverse("djinsight:record_page_view"),
            data=json.dumps(data),
            content_type="application/json",
        )

        self.assertEqual(response.status_code, 200)
        result = response.json()
        self.assertTrue(result.get("success"))

    def test_record_view_requires_object_id(self):
        """Test that object_id is required."""
        data = {
            "content_type": f"{self.content_type.app_label}.{self.content_type.model}",
            "url": "/test/",
        }

        response = self.client.post(
            reverse("djinsight:record_page_view"),
            data=json.dumps(data),
            content_type="application/json",
        )

        self.assertEqual(response.status_code, 400)

    def test_record_view_requires_content_type(self):
        """Test that content_type is required."""
        data = {
            "object_id": 1,
            "url": "/test/",
        }

        response = self.client.post(
            reverse("djinsight:record_page_view"),
            data=json.dumps(data),
            content_type="application/json",
        )

        self.assertEqual(response.status_code, 400)

    def test_record_view_ignores_unregistered_model(self):
        """Test that unregistered models are ignored."""
        # Unregister the model
        ContentTypeRegistry.objects.filter(content_type=self.content_type).delete()

        data = {
            "content_type": f"{self.content_type.app_label}.{self.content_type.model}",
            "object_id": 1,
            "url": "/test/",
        }

        response = self.client.post(
            reverse("djinsight:record_page_view"),
            data=json.dumps(data),
            content_type="application/json",
        )

        self.assertEqual(response.status_code, 200)
        result = response.json()
        self.assertEqual(result.get("status"), "ignored")


class MCPEndpointTest(TestCase):
    """Test MCP endpoint."""

    def setUp(self):
        """Set up test data."""
        self.client = Client()
        self.api_key = MCPAPIKey.create_key("test-key", "Test API key")
        self.content_type = ContentType.objects.get_for_model(PageViewStatistics)
        ContentTypeRegistry.register(PageViewStatistics)

    def test_mcp_requires_auth(self):
        """Test that MCP endpoint requires authentication."""
        response = self.client.post(
            "/djinsight/mcp/",
            data=json.dumps({"action": "list_tools"}),
            content_type="application/json",
        )

        self.assertEqual(response.status_code, 401)

    def test_mcp_list_tools(self):
        """Test MCP list_tools action."""
        response = self.client.post(
            "/djinsight/mcp/",
            data=json.dumps({"action": "list_tools"}),
            content_type="application/json",
            HTTP_AUTHORIZATION=f"Bearer {self.api_key.key}",
        )

        self.assertEqual(response.status_code, 200)
        result = response.json()
        self.assertIn("tools", result)
        self.assertIsInstance(result["tools"], list)
        self.assertGreater(len(result["tools"]), 0)

    def test_mcp_list_tracked_models(self):
        """Test MCP list_tracked_models tool."""
        response = self.client.post(
            "/djinsight/mcp/",
            data=json.dumps(
                {
                    "action": "execute_tool",
                    "tool_name": "list_tracked_models",
                    "arguments": {},
                }
            ),
            content_type="application/json",
            HTTP_AUTHORIZATION=f"Bearer {self.api_key.key}",
        )

        self.assertEqual(response.status_code, 200)
        result = response.json()
        self.assertIn("tracked_models", result)
        self.assertIsInstance(result["tracked_models"], list)

    def test_mcp_get_page_stats(self):
        """Test MCP get_page_stats tool."""
        # Create some stats
        PageViewStatistics.objects.create(
            content_type=self.content_type,
            object_id=1,
            total_views=10,
            unique_views=5,
        )

        response = self.client.post(
            "/djinsight/mcp/",
            data=json.dumps(
                {
                    "action": "execute_tool",
                    "tool_name": "get_page_stats",
                    "arguments": {
                        "content_type": f"{self.content_type.app_label}.{self.content_type.model}",
                        "object_id": 1,
                    },
                }
            ),
            content_type="application/json",
            HTTP_AUTHORIZATION=f"Bearer {self.api_key.key}",
        )

        self.assertEqual(response.status_code, 200)
        result = response.json()
        self.assertEqual(result["total_views"], 10)
        self.assertEqual(result["unique_views"], 5)

    def test_mcp_invalid_api_key(self):
        """Test MCP with invalid API key."""
        response = self.client.post(
            "/djinsight/mcp/",
            data=json.dumps({"action": "list_tools"}),
            content_type="application/json",
            HTTP_AUTHORIZATION="Bearer invalid-key",
        )

        self.assertEqual(response.status_code, 403)



"""Tests for djinsight models."""

from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.utils import timezone

from djinsight.models import (
    ContentTypeRegistry,
    MCPAPIKey,
    PageViewEvent,
    PageViewStatistics,
    PageViewSummary,
)


class ContentTypeRegistryTest(TestCase):
    """Test ContentTypeRegistry model."""

    def setUp(self):
        """Set up test data."""
        self.content_type = ContentType.objects.get_for_model(PageViewStatistics)

    def test_register_creates_entry(self):
        """Test that register creates a ContentTypeRegistry entry."""
        ContentTypeRegistry.register(PageViewStatistics)

        registry = ContentTypeRegistry.objects.get(content_type=self.content_type)
        self.assertTrue(registry.enabled)
        self.assertTrue(registry.track_anonymous)
        self.assertTrue(registry.track_authenticated)

    def test_register_with_custom_settings(self):
        """Test register with custom settings."""
        ContentTypeRegistry.register(
            PageViewStatistics,
            track_anonymous=False,
            track_authenticated=True,
        )

        registry = ContentTypeRegistry.objects.get(content_type=self.content_type)
        self.assertFalse(registry.track_anonymous)
        self.assertTrue(registry.track_authenticated)

    def test_is_tracked_returns_true_for_registered(self):
        """Test is_tracked returns True for registered model."""
        ContentTypeRegistry.register(PageViewStatistics)

        obj = PageViewStatistics.objects.create(
            content_type=self.content_type, object_id=1
        )
        self.assertTrue(ContentTypeRegistry.is_tracked(obj))

    def test_is_tracked_returns_false_for_unregistered(self):
        """Test is_tracked returns False for unregistered model."""
        obj = PageViewStatistics.objects.create(
            content_type=self.content_type, object_id=1
        )
        self.assertFalse(ContentTypeRegistry.is_tracked(obj))

    def test_is_tracked_returns_false_for_disabled(self):
        """Test is_tracked returns False for disabled registry."""
        ContentTypeRegistry.register(PageViewStatistics)
        registry = ContentTypeRegistry.objects.get(content_type=self.content_type)
        registry.enabled = False
        registry.save()

        obj = PageViewStatistics.objects.create(
            content_type=self.content_type, object_id=1
        )
        self.assertFalse(ContentTypeRegistry.is_tracked(obj))


class PageViewEventTest(TestCase):
    """Test PageViewEvent model."""

    def setUp(self):
        """Set up test data."""
        self.content_type = ContentType.objects.get_for_model(PageViewStatistics)

    def test_create_event(self):
        """Test creating a PageViewEvent."""
        event = PageViewEvent.objects.create(
            content_type=self.content_type,
            object_id=1,
            url="/test/",
            session_key="test-session",
            ip_address="127.0.0.1",
            user_agent="Test Agent",
            referrer="https://example.com",
            is_unique=True,
        )

        self.assertEqual(event.object_id, 1)
        self.assertEqual(event.url, "/test/")
        self.assertEqual(event.session_key, "test-session")
        self.assertTrue(event.is_unique)
        self.assertIsNotNone(event.timestamp)

    def test_event_str_representation(self):
        """Test string representation of PageViewEvent."""
        event = PageViewEvent.objects.create(
            content_type=self.content_type,
            object_id=1,
            url="/test/",
        )

        # Format: "{content_type} #{object_id} at {timestamp}"
        self.assertIn(str(self.content_type), str(event))
        self.assertIn("#1", str(event))
        self.assertIn("at", str(event))


class PageViewStatisticsTest(TestCase):
    """Test PageViewStatistics model."""

    def setUp(self):
        """Set up test data."""
        self.content_type = ContentType.objects.get_for_model(PageViewStatistics)

    def test_create_statistics(self):
        """Test creating PageViewStatistics."""
        stats = PageViewStatistics.objects.create(
            content_type=self.content_type,
            object_id=1,
            total_views=10,
            unique_views=5,
        )

        self.assertEqual(stats.total_views, 10)
        self.assertEqual(stats.unique_views, 5)
        self.assertIsNone(stats.first_viewed_at)
        self.assertIsNone(stats.last_viewed_at)

    def test_get_for_object(self):
        """Test get_for_object class method."""
        # Create stats for PageViewStatistics model itself
        stats = PageViewStatistics.objects.create(
            content_type=self.content_type,
            object_id=1,
            total_views=10,
        )

        # Use the stats object itself as test object
        retrieved = PageViewStatistics.get_for_object(stats)
        self.assertIsNotNone(retrieved)
        # Should find stats for PageViewStatistics with same pk
        self.assertEqual(retrieved.object_id, stats.pk)

    def test_stats_str_representation(self):
        """Test string representation of PageViewStatistics."""
        stats = PageViewStatistics.objects.create(
            content_type=self.content_type,
            object_id=1,
            total_views=10,
        )

        # Format: "{content_type} #{object_id}: {total_views} views"
        self.assertIn(str(self.content_type), str(stats))
        self.assertIn("#1", str(stats))
        self.assertIn("10 views", str(stats))


class PageViewSummaryTest(TestCase):
    """Test PageViewSummary model."""

    def setUp(self):
        """Set up test data."""
        self.content_type = ContentType.objects.get_for_model(PageViewStatistics)

    def test_create_summary(self):
        """Test creating a PageViewSummary."""
        today = timezone.now().date()
        summary = PageViewSummary.objects.create(
            content_type=self.content_type,
            object_id=1,
            date=today,
            total_views=100,
            unique_views=75,
        )

        self.assertEqual(summary.total_views, 100)
        self.assertEqual(summary.unique_views, 75)
        self.assertEqual(summary.date, today)

    def test_summary_str_representation(self):
        """Test string representation of PageViewSummary."""
        today = timezone.now().date()
        summary = PageViewSummary.objects.create(
            content_type=self.content_type,
            object_id=1,
            date=today,
            total_views=100,
        )

        # Format: "{content_type} #{object_id} - {date}: {total_views} views"
        self.assertIn(str(self.content_type), str(summary))
        self.assertIn("#1", str(summary))
        self.assertIn(str(today), str(summary))
        self.assertIn("100 views", str(summary))


class MCPAPIKeyTest(TestCase):
    """Test MCPAPIKey model."""

    def test_create_key(self):
        """Test creating an API key."""
        key = MCPAPIKey.create_key("test-key", "Test description")

        self.assertEqual(key.name, "test-key")
        self.assertEqual(key.description, "Test description")
        self.assertTrue(key.is_active)
        self.assertIsNotNone(key.key)
        self.assertEqual(len(key.key), 64)

    def test_validate_key_returns_true_for_valid(self):
        """Test validate_key returns True for valid key."""
        key = MCPAPIKey.create_key("test-key")
        self.assertTrue(MCPAPIKey.validate_key(key.key))

    def test_validate_key_returns_false_for_invalid(self):
        """Test validate_key returns False for invalid key."""
        self.assertFalse(MCPAPIKey.validate_key("invalid-key"))

    def test_validate_key_returns_false_for_inactive(self):
        """Test validate_key returns False for inactive key."""
        key = MCPAPIKey.create_key("test-key")
        key.is_active = False
        key.save()

        self.assertFalse(MCPAPIKey.validate_key(key.key))

    def test_validate_key_updates_last_used(self):
        """Test that validate_key updates last_used_at."""
        key = MCPAPIKey.create_key("test-key")
        original_last_used = key.last_used_at

        MCPAPIKey.validate_key(key.key)

        key.refresh_from_db()
        self.assertIsNotNone(key.last_used_at)
        if original_last_used:
            self.assertGreater(key.last_used_at, original_last_used)

    def test_generate_key_creates_unique_key(self):
        """Test that generate_key creates unique keys."""
        key1 = MCPAPIKey.generate_key()
        key2 = MCPAPIKey.generate_key()

        self.assertNotEqual(key1, key2)
        self.assertEqual(len(key1), 64)
        self.assertEqual(len(key2), 64)



import json
import logging
from typing import Any, Dict

import redis
from django.conf import settings
from redis.exceptions import ConnectionError, TimeoutError

from djinsight.conf import djinsight_settings
from djinsight.providers.base import BaseProvider

logger = logging.getLogger(__name__)


class RedisProvider(BaseProvider):

    def __init__(self):
        self.client = self._get_redis_client()
        self.key_prefix = djinsight_settings.REDIS_KEY_PREFIX

    def _get_redis_client(self):
        try:
            client = redis.Redis(
                host=djinsight_settings.REDIS_HOST,
                port=djinsight_settings.REDIS_PORT,
                db=djinsight_settings.REDIS_DB,
                password=djinsight_settings.REDIS_PASSWORD,
                socket_timeout=djinsight_settings.REDIS_TIMEOUT,
                socket_connect_timeout=djinsight_settings.REDIS_CONNECT_TIMEOUT,
                health_check_interval=30,
            )
            client.ping()
            logger.info("Redis connection established")
            return client
        except (ConnectionError, TimeoutError) as e:
            logger.error(f"Redis connection failed: {e}")
            return None

    async def record_view(self, event_data: Dict[str, Any]) -> Dict[str, Any]:
        if not self.client:
            return {'status': 'error', 'message': 'Redis unavailable'}

        try:
            view_id = event_data['view_id']
            content_type = event_data['content_type']
            object_id = event_data['object_id']
            session_key = event_data['session_key']
            is_unique = event_data['is_unique']

            expiration = djinsight_settings.REDIS_EXPIRATION

            pipe = self.client.pipeline()
            pipe.setex(f"{self.key_prefix}{view_id}", expiration, json.dumps(event_data))
            pipe.incr(f"{self.key_prefix}counter:{content_type}:{object_id}")

            if is_unique:
                session_key_redis = f"{self.key_prefix}session:{session_key}:page:{content_type}:{object_id}"
                pipe.setex(session_key_redis, expiration, 1)
                pipe.incr(f"{self.key_prefix}unique_counter:{content_type}:{object_id}")

            pipe.execute()

            return {'status': 'success', 'view_id': view_id, 'is_unique': is_unique}

        except Exception as e:
            logger.error(f"Error recording view in Redis: {e}")
            return {'status': 'error', 'message': str(e)}

    async def get_stats(self, content_type: str, object_id: int) -> Dict[str, Any]:
        if not self.client:
            return {'total_views': 0, 'unique_views': 0}

        try:
            total_views = self.client.get(f"{self.key_prefix}counter:{content_type}:{object_id}")
            unique_views = self.client.get(f"{self.key_prefix}unique_counter:{content_type}:{object_id}")

            return {
                'total_views': int(total_views) if total_views else 0,
                'unique_views': int(unique_views) if unique_views else 0,
            }
        except Exception as e:
            logger.error(f"Error getting stats from Redis: {e}")
            return {'total_views': 0, 'unique_views': 0}

    async def increment_counter(self, key: str, amount: int = 1) -> int:
        if not self.client:
            return 0

        try:
            return self.client.incr(f"{self.key_prefix}{key}", amount)
        except Exception as e:
            logger.error(f"Error incrementing counter: {e}")
            return 0

    async def check_unique_view(self, session_key: str, content_type: str, object_id: int) -> bool:
        if not self.client:
            return False

        try:
            key = f"{self.key_prefix}session:{session_key}:page:{content_type}:{object_id}"
            return not self.client.exists(key)
        except Exception as e:
            logger.error(f"Error checking unique view: {e}")
            return False

    async def mark_viewed(self, session_key: str, content_type: str, object_id: int, ttl: int) -> None:
        if not self.client:
            return

        try:
            key = f"{self.key_prefix}session:{session_key}:page:{content_type}:{object_id}"
            self.client.setex(key, ttl, 1)
        except Exception as e:
            logger.error(f"Error marking viewed: {e}")



# Generated by Django 5.2.2 on 2025-06-07 15:13

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="PageViewLog",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "page_id",
                    models.PositiveIntegerField(
                        db_index=True,
                        help_text="ID of the viewed object",
                        verbose_name="Object ID",
                    ),
                ),
                (
                    "content_type",
                    models.CharField(
                        db_index=True,
                        help_text="Content type of the viewed object",
                        max_length=100,
                        verbose_name="Content Type",
                    ),
                ),
                (
                    "url",
                    models.CharField(
                        help_text="URL of the viewed object",
                        max_length=500,
                        verbose_name="URL",
                    ),
                ),
                (
                    "session_key",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        help_text="Session key of the visitor",
                        max_length=40,
                        null=True,
                        verbose_name="Session Key",
                    ),
                ),
                (
                    "ip_address",
                    models.GenericIPAddressField(
                        blank=True,
                        help_text="IP address of the visitor",
                        null=True,
                        verbose_name="IP Address",
                    ),
                ),
                (
                    "user_agent",
                    models.TextField(
                        blank=True,
                        help_text="User agent of the visitor",
                        null=True,
                        verbose_name="User Agent",
                    ),
                ),
                (
                    "referrer",
                    models.URLField(
                        blank=True,
                        help_text="Referrer URL",
                        max_length=500,
                        null=True,
                        verbose_name="Referrer",
                    ),
                ),
                (
                    "timestamp",
                    models.DateTimeField(
                        db_index=True,
                        default=django.utils.timezone.now,
                        help_text="When the object was viewed",
                        verbose_name="Timestamp",
                    ),
                ),
                (
                    "is_unique",
                    models.BooleanField(
                        default=False,
                        help_text="Whether this is a unique view for this session",
                        verbose_name="Is Unique",
                    ),
                ),
            ],
            options={
                "verbose_name": "Page View Log",
                "verbose_name_plural": "Page View Logs",
                "ordering": ["-timestamp"],
                "indexes": [
                    models.Index(
                        fields=["page_id", "timestamp"],
                        name="djinsight_p_page_id_a3ba77_idx",
                    ),
                    models.Index(
                        fields=["content_type", "timestamp"],
                        name="djinsight_p_content_7eddbf_idx",
                    ),
                    models.Index(
                        fields=["session_key", "page_id"],
                        name="djinsight_p_session_b10779_idx",
                    ),
                    models.Index(
                        fields=["timestamp"], name="djinsight_p_timesta_9d0cb4_idx"
                    ),
                ],
            },
        ),
        migrations.CreateModel(
            name="PageViewSummary",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "page_id",
                    models.PositiveIntegerField(
                        db_index=True,
                        help_text="ID of the viewed object",
                        verbose_name="Object ID",
                    ),
                ),
                (
                    "content_type",
                    models.CharField(
                        help_text="Content type of the viewed object",
                        max_length=100,
                        verbose_name="Content Type",
                    ),
                ),
                (
                    "date",
                    models.DateField(
                        db_index=True,
                        help_text="Date of the summary",
                        verbose_name="Date",
                    ),
                ),
                (
                    "total_views",
                    models.PositiveIntegerField(
                        default=0,
                        help_text="Total views for this day",
                        verbose_name="Total Views",
                    ),
                ),
                (
                    "unique_views",
                    models.PositiveIntegerField(
                        default=0,
                        help_text="Unique views for this day",
                        verbose_name="Unique Views",
                    ),
                ),
            ],
            options={
                "verbose_name": "Page View Summary",
                "verbose_name_plural": "Page View Summaries",
                "ordering": ["-date"],
                "indexes": [
                    models.Index(
                        fields=["page_id", "date"],
                        name="djinsight_p_page_id_cd69db_idx",
                    ),
                    models.Index(
                        fields=["content_type", "date"],
                        name="djinsight_p_content_07ac79_idx",
                    ),
                    models.Index(fields=["date"], name="djinsight_p_date_fbf615_idx"),
                ],
                "unique_together": {("page_id", "date")},
            },
        ),
    ]



from datetime import datetime
from typing import Any, Dict

from asgiref.sync import sync_to_async
from django.contrib.contenttypes.models import ContentType
from django.utils import timezone

from djinsight.models import PageViewEvent, PageViewStatistics
from djinsight.providers.base import AsyncBaseProvider, BaseProvider


class DatabaseProvider(BaseProvider):
    """
    Synchronous database provider.
    Writes page views directly to database without Redis buffering.
    Use when you don't want Redis/Celery dependencies.
    """

    def record_view(self, event_data: Dict[str, Any]) -> Dict[str, Any]:
        """Record a page view directly to database."""
        try:
            content_type_str = event_data.get("content_type")
            object_id = event_data.get("object_id")

            app_label, model = content_type_str.split(".")
            ct = ContentType.objects.get_by_natural_key(app_label, model.lower())

            timestamp = event_data.get("timestamp")
            if isinstance(timestamp, (int, float)):
                timestamp = datetime.fromtimestamp(
                    timestamp, tz=timezone.get_current_timezone()
                )
            elif not timestamp:
                timestamp = timezone.now()

            event = PageViewEvent.objects.create(
                content_type=ct,
                object_id=object_id,
                url=event_data.get("url", ""),
                session_key=event_data.get("session_key", "")[:255],
                ip_address=event_data.get("ip_address", ""),
                user_agent=event_data.get("user_agent", "")[:1000],
                referrer=event_data.get("referrer", "")[:500],
                timestamp=timestamp,
                is_unique=event_data.get("is_unique", False),
            )

            stats, created = PageViewStatistics.objects.get_or_create(
                content_type=ct,
                object_id=object_id,
            )

            stats.total_views += 1
            if event_data.get("is_unique"):
                stats.unique_views += 1

            stats.last_viewed_at = timezone.now()
            if not stats.first_viewed_at:
                stats.first_viewed_at = timezone.now()

            stats.save(
                update_fields=[
                    "total_views",
                    "unique_views",
                    "last_viewed_at",
                    "first_viewed_at",
                ]
            )

            return {
                "success": True,
                "event_id": event.id,
                "stats": {
                    "total_views": stats.total_views,
                    "unique_views": stats.unique_views,
                },
            }

        except Exception as e:
            return {"success": False, "error": str(e)}

    def get_stats(self, content_type: str, object_id: int) -> Dict[str, Any]:
        """Get statistics from database."""
        try:
            app_label, model = content_type.split(".")
            ct = ContentType.objects.get_by_natural_key(app_label, model.lower())

            stats = PageViewStatistics.objects.filter(
                content_type=ct, object_id=object_id
            ).first()

            if not stats:
                return {
                    "total_views": 0,
                    "unique_views": 0,
                    "first_viewed_at": None,
                    "last_viewed_at": None,
                }

            return {
                "total_views": stats.total_views,
                "unique_views": stats.unique_views,
                "first_viewed_at": stats.first_viewed_at.isoformat()
                if stats.first_viewed_at
                else None,
                "last_viewed_at": stats.last_viewed_at.isoformat()
                if stats.last_viewed_at
                else None,
            }

        except Exception as e:
            return {"error": str(e)}

    def check_unique_view(
        self, session_key: str, content_type: str, object_id: int
    ) -> bool:
        """Check if this is a unique view by checking existing events."""
        try:
            app_label, model = content_type.split(".")
            ct = ContentType.objects.get_by_natural_key(app_label, model.lower())

            exists = PageViewEvent.objects.filter(
                content_type=ct,
                object_id=object_id,
                session_key=session_key,
            ).exists()

            return not exists
        except Exception:
            return True

    def increment_counter(self, key: str, amount: int = 1) -> int:
        """Not used in database provider - stats are updated directly."""
        return amount

    def mark_viewed(
        self, session_key: str, content_type: str, object_id: int, ttl: int = 86400
    ) -> None:
        """Not needed for database provider - uniqueness tracked via PageViewEvent."""
        pass


class AsyncDatabaseProvider(AsyncBaseProvider):
    """
    Asynchronous database provider.
    Wraps DatabaseProvider methods with sync_to_async for use in async views.
    Use when you need async support without Redis.
    """

    def __init__(self):
        self._sync_provider = DatabaseProvider()

    async def record_view(self, event_data: Dict[str, Any]) -> Dict[str, Any]:
        """Record a page view asynchronously."""
        return await sync_to_async(
            self._sync_provider.record_view, thread_sensitive=True
        )(event_data)

    async def get_stats(self, content_type: str, object_id: int) -> Dict[str, Any]:
        """Get statistics asynchronously."""
        return await sync_to_async(
            self._sync_provider.get_stats, thread_sensitive=True
        )(content_type, object_id)

    async def check_unique_view(
        self, session_key: str, content_type: str, object_id: int
    ) -> bool:
        """Check if this is a unique view asynchronously."""
        return await sync_to_async(
            self._sync_provider.check_unique_view, thread_sensitive=True
        )(session_key, content_type, object_id)

    async def increment_counter(self, key: str, amount: int = 1) -> int:
        """Not used in database provider."""
        return amount

    async def mark_viewed(
        self, session_key: str, content_type: str, object_id: int, ttl: int = 86400
    ) -> None:
        """Not needed for database provider."""
        pass



import json
import logging
from datetime import datetime, timedelta

from django.apps import apps
from django.contrib.contenttypes.models import ContentType
from django.db import transaction
from django.utils import timezone
from redis.exceptions import ConnectionError, TimeoutError

from djinsight.conf import djinsight_settings
from djinsight.models import PageViewEvent, PageViewStatistics, PageViewSummary
from djinsight.providers.redis import RedisProvider

logger = logging.getLogger(__name__)

redis_provider = RedisProvider()
redis_client = redis_provider.client
REDIS_KEY_PREFIX = djinsight_settings.REDIS_KEY_PREFIX

# Try to import Celery - if not available, tasks will be regular functions
try:
    from celery import shared_task

    HAS_CELERY = True
except ImportError:
    # Fallback decorator for when Celery is not available
    def shared_task(func):
        return func

    HAS_CELERY = False


@shared_task(
    bind=True,
    max_retries=3,
    default_retry_delay=60,
    task_time_limit=djinsight_settings.PROCESS_TASK_TIME_LIMIT,
    task_soft_time_limit=djinsight_settings.PROCESS_TASK_SOFT_TIME_LIMIT,
)
def process_page_views_task(
    self,
    batch_size=None,
    max_records=None,
):
    """
    Celery task to process page views from Redis and store them in the database.

    Args:
        batch_size (int): Number of records to process in a single transaction
        max_records (int): Maximum number of records to process in a single run

    Returns:
        int: Number of records processed
    """
    try:
        batch_size = batch_size or djinsight_settings.PROCESS_BATCH_SIZE
        max_records = max_records or djinsight_settings.PROCESS_MAX_RECORDS
        return process_page_views(batch_size, max_records)
    except Exception as exc:
        logger.error(f"Error processing page views: {exc}")
        if HAS_CELERY:
            raise self.retry(exc=exc)
        else:
            raise


@shared_task(
    bind=True,
    max_retries=3,
    default_retry_delay=60,
    task_time_limit=djinsight_settings.SUMMARY_TASK_TIME_LIMIT,
    task_soft_time_limit=djinsight_settings.SUMMARY_TASK_SOFT_TIME_LIMIT,
)
def generate_daily_summaries_task(
    self, days_back=None
):
    """
    Celery task to generate daily page view summaries.

    Args:
        days_back (int): Number of days back to process

    Returns:
        int: Number of summaries generated
    """
    try:
        days_back = days_back or djinsight_settings.SUMMARY_DAYS_BACK
        return generate_daily_summaries(days_back)
    except Exception as exc:
        logger.error(f"Error generating daily summaries: {exc}")
        if HAS_CELERY:
            raise self.retry(exc=exc)
        else:
            raise


@shared_task(
    bind=True,
    max_retries=3,
    default_retry_delay=60,
    task_time_limit=djinsight_settings.CLEANUP_TASK_TIME_LIMIT,
    task_soft_time_limit=djinsight_settings.CLEANUP_TASK_SOFT_TIME_LIMIT,
)
def cleanup_old_data_task(
    self, days_to_keep=None
):
    """
    Celery task to cleanup old page view logs.

    Args:
        days_to_keep (int): Number of days of logs to keep

    Returns:
        int: Number of records deleted
    """
    try:
        days_to_keep = days_to_keep or djinsight_settings.CLEANUP_DAYS_TO_KEEP
        return cleanup_old_data(days_to_keep)
    except Exception as exc:
        logger.error(f"Error cleaning up old data: {exc}")
        if HAS_CELERY:
            raise self.retry(exc=exc)
        else:
            raise


def process_page_views(
    batch_size=None,
    max_records=None,
):
    """
    Process page views from Redis and store them in the database.

    Args:
        batch_size (int): Number of records to process in a single transaction
        max_records (int): Maximum number of records to process in a single run

    Returns:
        int: Number of records processed
    """
    batch_size = batch_size or djinsight_settings.PROCESS_BATCH_SIZE
    max_records = max_records or djinsight_settings.PROCESS_MAX_RECORDS
    if not redis_client:
        logger.error("Redis client not available")
        return 0

    logger.info("Starting to process page views from Redis")

    try:
        # Get all keys matching the page view pattern, excluding counters and sessions
        pattern = f"{REDIS_KEY_PREFIX}*"
        exclude_patterns = [
            f"{REDIS_KEY_PREFIX}counter:",
            f"{REDIS_KEY_PREFIX}unique_counter:",
            f"{REDIS_KEY_PREFIX}session:",
        ]

        # Get all keys
        all_keys = redis_client.keys(pattern)
        # Filter out counter and session keys
        keys = [
            key.decode("utf-8")
            for key in all_keys
            if not any(
                key.decode("utf-8").startswith(exclude) for exclude in exclude_patterns
            )
        ]

        # Limit the number of keys to process
        keys = keys[:max_records]

        if not keys:
            logger.info("No page views to process")
            return 0

        logger.info(f"Found {len(keys)} page views to process")

        # Process in batches
        processed_count = 0
        for i in range(0, len(keys), batch_size):
            batch_keys = keys[i : i + batch_size]
            batch_processed = process_batch(batch_keys)
            processed_count += batch_processed

            # Log progress
            if i % (batch_size * 10) == 0:
                logger.info(f"Processed {processed_count} / {len(keys)} page views")

        logger.info(f"Completed processing {processed_count} page views")
        return processed_count

    except (ConnectionError, TimeoutError) as e:
        logger.error(f"Redis connection error: {e}")
        raise
    except Exception as e:
        logger.error(f"Error processing page views: {e}")
        raise


def process_batch(keys):
    """
    Process a batch of page views.

    Args:
        keys (list): List of Redis keys to process

    Returns:
        int: Number of records processed in this batch
    """
    if not redis_client:
        return 0

    pipe = redis_client.pipeline()
    for key in keys:
        pipe.get(key)
    values = pipe.execute()

    page_view_events = []
    page_view_counters = {}
    processed_count = 0

    for key, value in zip(keys, values):
        if value is None:
            continue

        try:
            data = json.loads(value.decode("utf-8"))

            # Extract data with validation
            page_id = data.get("page_id")
            content_type = data.get("content_type")
            url = data.get("url")
            session_key = data.get("session_key")
            ip_address = data.get("ip_address")
            user_agent = data.get("user_agent")
            referrer = data.get("referrer")
            timestamp_value = data.get("timestamp")
            is_unique = data.get("is_unique", False)

            # Skip if missing essential data
            if not all([page_id, content_type, url]):
                logger.warning(f"Skipping incomplete page view data in key {key}")
                continue

            # Convert timestamp
            if timestamp_value:
                if isinstance(timestamp_value, str):
                    timestamp = datetime.fromtimestamp(
                        int(timestamp_value), tz=timezone.get_current_timezone()
                    )
                else:
                    timestamp = datetime.fromtimestamp(
                        timestamp_value, tz=timezone.get_current_timezone()
                    )
            else:
                timestamp = timezone.now()

            app_label, model = content_type.split(".")
            ct = ContentType.objects.get_by_natural_key(app_label, model)

            page_view_events.append(
                PageViewEvent(
                    content_type=ct,
                    object_id=page_id,
                    url=url,
                    session_key=session_key[:255] if session_key else "",
                    ip_address=ip_address,
                    user_agent=user_agent[:1000] if user_agent else "",
                    referrer=referrer[:500] if referrer else "",
                    timestamp=timestamp,
                    is_unique=is_unique,
                )
            )

            counter_key = (ct.id, page_id)
            if counter_key not in page_view_counters:
                page_view_counters[counter_key] = (1, 1 if is_unique else 0)
            else:
                total, unique = page_view_counters[counter_key]
                page_view_counters[counter_key] = (
                    total + 1,
                    unique + (1 if is_unique else 0),
                )

            processed_count += 1

        except (json.JSONDecodeError, ValueError, TypeError) as e:
            logger.error(f"Error processing page view {key}: {e}")
            continue
        except Exception as e:
            logger.error(f"Unexpected error processing page view {key}: {e}")
            continue

    if page_view_events or page_view_counters:
        with transaction.atomic():
            if page_view_events:
                PageViewEvent.objects.bulk_create(page_view_events, batch_size=500)

            for (content_type_id, object_id), (total, unique) in page_view_counters.items():
                try:
                    stats, created = PageViewStatistics.objects.get_or_create(
                        content_type_id=content_type_id,
                        object_id=object_id,
                    )
                    stats.total_views += total
                    stats.unique_views += unique
                    stats.last_viewed_at = timezone.now()
                    if not stats.first_viewed_at:
                        stats.first_viewed_at = timezone.now()
                    stats.save(update_fields=['total_views', 'unique_views', 'last_viewed_at', 'first_viewed_at'])

                except Exception as e:
                    logger.error(
                        f"Error updating page statistics for content_type {content_type_id} object {object_id}: {e}"
                    )

    # Delete processed keys from Redis
    if keys:
        try:
            redis_client.delete(*keys)
        except Exception as e:
            logger.error(f"Error deleting processed keys from Redis: {e}")

    return processed_count


def generate_daily_summaries(days_back=None):
    """
    Generate daily page view summaries from detailed logs.

    Args:
        days_back (int): Number of days back to process

    Returns:
        int: Number of summaries generated
    """
    days_back = days_back or djinsight_settings.SUMMARY_DAYS_BACK
    logger.info(f"Generating daily summaries for the last {days_back} days")

    end_date = timezone.now().date()
    start_date = end_date - timedelta(days=days_back)

    summaries_created = 0

    page_views = (
        PageViewEvent.objects.filter(
            timestamp__date__gte=start_date, timestamp__date__lte=end_date
        )
        .values("content_type", "object_id", "timestamp__date")
        .distinct()
    )

    for view_data in page_views:
        content_type = view_data["content_type"]
        object_id = view_data["object_id"]
        date = view_data["timestamp__date"]

        date_views = PageViewEvent.objects.filter(
            content_type=content_type,
            object_id=object_id,
            timestamp__date=date
        )

        total_views = date_views.count()
        unique_views = date_views.values("session_key").distinct().count()

        summary, created = PageViewSummary.objects.update_or_create(
            content_type=content_type,
            object_id=object_id,
            date=date,
            defaults={
                "total_views": total_views,
                "unique_views": unique_views,
            },
        )

        if created:
            summaries_created += 1

    logger.info(f"Generated {summaries_created} daily summaries")
    return summaries_created


def cleanup_old_data(days_to_keep=None):
    """
    Cleanup old page view logs older than specified days.

    Args:
        days_to_keep (int): Number of days of logs to keep

    Returns:
        int: Number of records deleted
    """
    days_to_keep = days_to_keep or djinsight_settings.CLEANUP_DAYS_TO_KEEP
    cutoff_date = timezone.now() - timedelta(days=days_to_keep)

    logger.info(f"Cleaning up page view logs older than {cutoff_date}")

    deleted_count, _ = PageViewEvent.objects.filter(timestamp__lt=cutoff_date).delete()

    logger.info(f"Deleted {deleted_count} old page view events")

    # Also cleanup old Redis session keys (this is optional)
    if redis_client:
        try:
            # Clean up session keys older than days_to_keep
            session_pattern = f"{REDIS_KEY_PREFIX}session:*"
            session_keys = redis_client.keys(session_pattern)

            # Check TTL and delete expired keys manually
            # (Redis should handle this automatically, but just in case)
            deleted_sessions = 0
            for key in session_keys:
                ttl = redis_client.ttl(key)
                if ttl == -1:  # Key exists but has no expiration
                    redis_client.delete(key)
                    deleted_sessions += 1

            if deleted_sessions > 0:
                logger.info(
                    f"Cleaned up {deleted_sessions} orphaned session keys from Redis"
                )

        except Exception as e:
            logger.error(f"Error cleaning up Redis session keys: {e}")

    return deleted_count


def run_process_page_views(verbosity=1, **options):
    """Function that can be called from management command"""
    batch_size = options.get("batch_size") or djinsight_settings.PROCESS_BATCH_SIZE
    max_records = options.get("max_records") or djinsight_settings.PROCESS_MAX_RECORDS

    if verbosity >= 1:
        print(
            f"Processing page views with batch_size={batch_size}, max_records={max_records}"
        )

    processed = process_page_views(batch_size, max_records)

    if verbosity >= 1:
        print(f"Processed {processed} page views")

    return processed


def run_generate_summaries(verbosity=1, **options):
    """Function that can be called from management command"""
    days_back = options.get("days_back") or djinsight_settings.SUMMARY_DAYS_BACK

    if verbosity >= 1:
        print(f"Generating daily summaries for the last {days_back} days")

    generated = generate_daily_summaries(days_back)

    if verbosity >= 1:
        print(f"Generated {generated} daily summaries")

    return generated


def run_cleanup_old_data(verbosity=1, **options):
    """Function that can be called from management command"""
    days_to_keep = options.get("days_to_keep") or djinsight_settings.CLEANUP_DAYS_TO_KEEP

    if verbosity >= 1:
        print(f"Cleaning up data older than {days_to_keep} days")

    deleted = cleanup_old_data(days_to_keep)

    if verbosity >= 1:
        print(f"Deleted {deleted} old records")

    return deleted



"""
Celery configuration for djinsight.

This module provides Celery configuration and periodic tasks for processing
page view statistics.
"""

import os

from celery import Celery
from celery.schedules import crontab
from django.conf import settings

from djinsight.conf import djinsight_settings

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "your_project.settings")

app = Celery("djinsight")

app.config_from_object("django.conf:settings", namespace="CELERY")

app.autodiscover_tasks()


# Helper function to parse cron schedule from environment variables
def get_schedule_from_env(env_var, default_schedule):
    """
    Parse schedule from environment variable or return default.

    Environment variable can be:
    - "10" for every 10 seconds
    - "*/5" for every 5 minutes (cron format)
    - "0 1 * * *" for full cron expression
    """
    schedule_str = os.environ.get(env_var, None)
    if not schedule_str:
        return default_schedule

    # If it's just a number, treat as seconds
    if schedule_str.isdigit():
        return int(schedule_str)

    # If it contains spaces, treat as full cron expression
    if " " in schedule_str:
        parts = schedule_str.split()
        if len(parts) == 5:
            minute, hour, day, month, day_of_week = parts
            return crontab(
                minute=minute,
                hour=hour,
                day_of_month=day,
                month_of_year=month,
                day_of_week=day_of_week,
            )

    # If it contains */, treat as minute interval
    if "*/" in schedule_str:
        return crontab(minute=schedule_str)

    # If it's a single number without *, treat as minute interval
    if schedule_str.isdigit():
        return crontab(minute=f"*/{schedule_str}")

    return default_schedule


# Periodic tasks configuration with environment variable support
app.conf.beat_schedule = {
    "process-page-views": {
        "task": "djinsight.tasks.process_page_views_task",
        "schedule": get_schedule_from_env(
            "DJINSIGHT_PROCESS_SCHEDULE",
            10,
        ),
        "kwargs": {
            "batch_size": djinsight_settings.PROCESS_BATCH_SIZE,
            "max_records": djinsight_settings.PROCESS_MAX_RECORDS,
        },
    },
    "generate-daily-summaries": {
        "task": "djinsight.tasks.generate_daily_summaries_task",
        "schedule": get_schedule_from_env(
            "DJINSIGHT_SUMMARIES_SCHEDULE",
            crontab(minute="*/10"),
        ),
        "kwargs": {
            "days_back": djinsight_settings.SUMMARY_DAYS_BACK,
        },
    },
    "cleanup-old-data": {
        "task": "djinsight.tasks.cleanup_old_data_task",
        "schedule": get_schedule_from_env(
            "DJINSIGHT_CLEANUP_SCHEDULE",
            crontab(hour=1, minute=0),
        ),
        "kwargs": {
            "days_to_keep": djinsight_settings.CLEANUP_DAYS_TO_KEEP,
        },
    },
}

# Timezone configuration
app.conf.timezone = getattr(settings, "TIME_ZONE", "UTC")

# Additional Celery configuration
app.conf.update(
    # Task serialization
    task_serializer="json",
    accept_content=["json"],
    result_serializer="json",
    # Task routing
    task_routes={
        "djinsight.tasks.*": {"queue": "djinsight"},
    },
    # Task execution
    task_always_eager=getattr(settings, "CELERY_TASK_ALWAYS_EAGER", False),
    task_eager_propagates=True,
    # Worker configuration
    worker_prefetch_multiplier=1,
    task_acks_late=True,
    # Result backend (optional)
    result_backend=getattr(settings, "CELERY_RESULT_BACKEND", None),
    result_expires=3600,  # 1 hour
)


@app.task(bind=True)
def debug_task(self):
    """Debug task for testing Celery configuration."""
    print(f"Request: {self.request!r}")
    return "Debug task completed"


# Example of how to configure Celery in your Django settings:
"""
# Celery Configuration
CELERY_BROKER_URL = 'redis://localhost:6379/1'
CELERY_RESULT_BACKEND = 'redis://localhost:6379/1'

# djinsight specific settings
DJINSIGHT_BATCH_SIZE = 1000
DJINSIGHT_MAX_RECORDS = 10000
DJINSIGHT_SUMMARY_DAYS_BACK = 7
DJINSIGHT_DAYS_TO_KEEP = 90

# Redis settings for djinsight
DJINSIGHT_REDIS_HOST = 'localhost'
DJINSIGHT_REDIS_PORT = 6379
DJINSIGHT_REDIS_DB = 0
DJINSIGHT_REDIS_PASSWORD = None
DJINSIGHT_REDIS_KEY_PREFIX = 'djinsight:pageview:'
DJINSIGHT_REDIS_EXPIRATION = 60 * 60 * 24 * 7  # 7 days

# Enable/disable tracking
DJINSIGHT_ENABLE_TRACKING = True

# Celery Schedule Configuration (Environment Variables)
# DJINSIGHT_PROCESS_SCHEDULE = "10"        # Every 10 seconds (default)
# DJINSIGHT_SUMMARIES_SCHEDULE = "*/10"    # Every 10 minutes (default)
# DJINSIGHT_CLEANUP_SCHEDULE = "0 1 * * *"  # Daily at 1:00 AM (default)
"""



# Generated by Django 4.2 on 2025-01-27

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "djinsight",
            "0002_rename_djinsight_p_page_id_a3ba77_idx_djinsight_p_page_id_f86134_idx_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="pageviewlog",
            name="session_key",
            field=models.CharField(
                blank=True,
                db_index=True,
                help_text="Session key of the visitor",
                max_length=255,
                null=True,
                verbose_name="Session Key",
            ),
        ),
    ]



"""
djinsight - Django/Wagtail analytics package for real-time page view tracking.
"""

__version__ = "0.1.8"
__author__ = "MDigital"
__email__ = "kontakt@mdigital.com.pl"
__license__ = "MIT"

default_app_config = "djinsight.apps.DjInsightConfig"



from typing import Any, Callable

from django.conf import settings
from django.utils.module_loading import import_string


class DjInsightSettings:
    DEFAULTS = {
        "ENABLE_TRACKING": True,
        "AUTO_INJECT_TRACKING": True,
        "ADMIN_ONLY": False,
        "REDIS_HOST": "localhost",
        "REDIS_PORT": 6379,
        "REDIS_DB": 0,
        "REDIS_PASSWORD": None,
        "REDIS_TIMEOUT": 5,
        "REDIS_CONNECT_TIMEOUT": 5,
        "REDIS_KEY_PREFIX": "djinsight:pageview:",
        "REDIS_EXPIRATION": 60 * 60 * 24 * 7,
        "TRACK_MODELS": [],
        "TRACK_ANONYMOUS": True,
        "TRACK_AUTHENTICATED": True,
        "TRACK_STAFF": True,
        "TRACK_SUPERUSER": True,
        "MIDDLEWARE_CLASS": "djinsight.middleware.TrackingMiddleware",
        "STATS_TAG_FUNCTION": "djinsight.templatetags.djinsight_tags.stats",
        "WIDGET_RENDERER": "djinsight.renderers.DefaultWidgetRenderer",
        "CHART_RENDERER": "djinsight.renderers.DefaultChartRenderer",
        "PROVIDER_CLASS": None,
        "REGISTRY_CLASS": "djinsight.registry.ProviderRegistry",
        "EVENT_PROCESSOR": "djinsight.processors.DefaultEventProcessor",
        "SESSION_TRACKER": "djinsight.trackers.SessionTracker",
        "IP_EXTRACTOR": "djinsight.utils.get_client_ip",
        "USER_AGENT_PARSER": "djinsight.utils.parse_user_agent",
        "USE_REDIS": True,
        "USE_CELERY": True,
        "USE_ASYNC": False,  # Use async provider (for async Django views)
        "ASYNC_PROCESSING": True,
        "RETENTION_DAYS": 365,
        "SUMMARY_RETENTION_DAYS": 730,
        "CLEANUP_BATCH_SIZE": 1000,
        "PROCESS_BATCH_SIZE": 100,
        "PROCESS_MAX_RECORDS": 10000,
        "PROCESS_TASK_TIME_LIMIT": 1800,
        "PROCESS_TASK_SOFT_TIME_LIMIT": 1500,
        "SUMMARY_TASK_TIME_LIMIT": 900,
        "SUMMARY_TASK_SOFT_TIME_LIMIT": 720,
        "CLEANUP_TASK_TIME_LIMIT": 3600,
        "CLEANUP_TASK_SOFT_TIME_LIMIT": 3300,
        "SUMMARY_DAYS_BACK": 7,
        "CLEANUP_DAYS_TO_KEEP": 90,
        "CACHE_TTL": 300,
        "ENABLE_CACHING": True,
        "CACHE_BACKEND": "default",
        "PRIVACY_MODE": False,
        "ANONYMIZE_IP": False,
        "STORE_USER_AGENT": True,
        "STORE_REFERRER": True,
        "CELERY_TASK_TIME_LIMIT": 600,
        "CELERY_TASK_SOFT_TIME_LIMIT": 540,
        "MCP_ENABLED": True,
        "MCP_ENDPOINT_URL": "/djinsight/mcp/",
        "MCP_API_KEY_HEADER": "Authorization",
        "MCP_RATE_LIMIT": 100,
        "MCP_RATE_LIMIT_PERIOD": 60,
    }

    def __getattr__(self, name: str) -> Any:
        if name not in self.DEFAULTS:
            raise AttributeError(f"Invalid setting: {name}")

        user_settings = getattr(settings, "DJINSIGHT", {})

        if name in user_settings:
            return user_settings[name]

        legacy_key = f"DJINSIGHT_{name}"
        if hasattr(settings, legacy_key):
            return getattr(settings, legacy_key)

        return self.DEFAULTS[name]

    def get_class(self, setting_name: str) -> Callable:
        class_path = getattr(self, setting_name)
        return import_string(class_path)

    def get_middleware_class(self):
        return self.get_class("MIDDLEWARE_CLASS")

    def get_provider_class(self):
        provider_class_path = getattr(self, "PROVIDER_CLASS")
        if provider_class_path is None:
            if self.USE_REDIS:
                return import_string("djinsight.providers.redis.RedisProvider")
            else:
                return import_string("djinsight.providers.database.DatabaseProvider")
        return import_string(provider_class_path)

    def get_registry_class(self):
        return self.get_class("REGISTRY_CLASS")

    def get_widget_renderer(self):
        return self.get_class("WIDGET_RENDERER")

    def get_chart_renderer(self):
        return self.get_class("CHART_RENDERER")

    def get_event_processor(self):
        return self.get_class("EVENT_PROCESSOR")

    def get_session_tracker(self):
        return self.get_class("SESSION_TRACKER")

    def get_ip_extractor(self):
        return self.get_class("IP_EXTRACTOR")

    def get_user_agent_parser(self):
        return self.get_class("USER_AGENT_PARSER")

    def should_track_user(self, user) -> bool:
        if not user:
            return self.TRACK_ANONYMOUS

        if not user.is_authenticated:
            return self.TRACK_ANONYMOUS

        if user.is_superuser and not self.TRACK_SUPERUSER:
            return False

        if user.is_staff and not self.TRACK_STAFF:
            return False

        return self.TRACK_AUTHENTICATED


djinsight_settings = DjInsightSettings()



from django.urls import path

from djinsight import views
from djinsight.mcp.server import mcp_endpoint

app_name = "djinsight"

urlpatterns = [
    path("record-view/", views.record_page_view, name="record_page_view"),
    path("page-stats/", views.get_page_stats, name="get_page_stats"),
    path("mcp/", mcp_endpoint, name="mcp_endpoint"),
]



from django.db import models
from django.urls import reverse


class Post(models.Model):
    title = models.CharField(max_length=200)
    content = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse("blog:post_detail", kwargs={"pk": self.pk})



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="djinsight",
    version="0.3.1",
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    description="MCP-first analytics for Django/Wagtail package for tracking page view statistics with Redis and async processing",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/krystianmagdziarz/djinsight",
    author="MDigital",
    author_email="kontakt@mdigital.com.pl",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Framework :: Django :: 5.0",
        "Framework :: Wagtail",
        "Framework :: Wagtail :: 3",
        "Framework :: Wagtail :: 4",
        "Framework :: Wagtail :: 5",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=[
        "Django>=3.2",
        "wagtail>=3.0",
        "redis>=4.0.0",
        "celery>=5.0.0",
        "django-redis>=5.0.0",
        "django-environ>=0.9.0",
    ],
    extras_require={
        "dev": [
            "pytest>=6.0",
            "pytest-django>=4.0",
            "black>=22.0",
            "flake8>=4.0",
        ],
    },
    python_requires=">=3.8",
    keywords="django wagtail analytics statistics page views redis celery",
)



"""Tests for djinsight providers."""

from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.utils import timezone

from djinsight.models import PageViewEvent, PageViewStatistics
from djinsight.providers.database import AsyncDatabaseProvider, DatabaseProvider
from djinsight.registry import ProviderRegistry


class DatabaseProviderTest(TestCase):
    """Test DatabaseProvider functionality."""

    def setUp(self):
        """Set up test data."""
        self.provider = DatabaseProvider()
        self.content_type = ContentType.objects.get_for_model(PageViewStatistics)

    def test_record_view_creates_event(self):
        """Test that record_view creates a PageViewEvent."""
        event_data = {
            "content_type": f"{self.content_type.app_label}.{self.content_type.model}",
            "object_id": 1,
            "url": "/test/",
            "session_key": "test-session",
            "ip_address": "127.0.0.1",
            "user_agent": "Test Agent",
            "referrer": "https://example.com",
            "timestamp": timezone.now().timestamp(),
            "is_unique": True,
        }

        result = self.provider.record_view(event_data)

        self.assertTrue(result["success"])
        self.assertIn("event_id", result)
        self.assertIn("stats", result)

        # Verify event was created
        event = PageViewEvent.objects.get(id=result["event_id"])
        self.assertEqual(event.object_id, 1)
        self.assertEqual(event.session_key, "test-session")
        self.assertTrue(event.is_unique)

    def test_record_view_updates_statistics(self):
        """Test that record_view updates PageViewStatistics."""
        event_data = {
            "content_type": f"{self.content_type.app_label}.{self.content_type.model}",
            "object_id": 1,
            "url": "/test/",
            "session_key": "test-session",
            "timestamp": timezone.now().timestamp(),
            "is_unique": True,
        }

        result = self.provider.record_view(event_data)

        stats = PageViewStatistics.objects.get(
            content_type=self.content_type, object_id=1
        )
        self.assertEqual(stats.total_views, 1)
        self.assertEqual(stats.unique_views, 1)
        self.assertIsNotNone(stats.first_viewed_at)
        self.assertIsNotNone(stats.last_viewed_at)

    def test_record_view_increments_counts(self):
        """Test that multiple views increment counts correctly."""
        event_data = {
            "content_type": f"{self.content_type.app_label}.{self.content_type.model}",
            "object_id": 1,
            "url": "/test/",
            "session_key": "test-session",
            "timestamp": timezone.now().timestamp(),
            "is_unique": True,
        }

        # First view (unique)
        self.provider.record_view(event_data)

        # Second view (not unique)
        event_data["is_unique"] = False
        event_data["session_key"] = "test-session-2"
        self.provider.record_view(event_data)

        stats = PageViewStatistics.objects.get(
            content_type=self.content_type, object_id=1
        )
        self.assertEqual(stats.total_views, 2)
        self.assertEqual(stats.unique_views, 1)

    def test_get_stats_returns_correct_data(self):
        """Test that get_stats returns correct statistics."""
        # Create some stats
        stats = PageViewStatistics.objects.create(
            content_type=self.content_type,
            object_id=1,
            total_views=10,
            unique_views=5,
            first_viewed_at=timezone.now(),
            last_viewed_at=timezone.now(),
        )

        result = self.provider.get_stats(
            f"{self.content_type.app_label}.{self.content_type.model}", 1
        )

        self.assertEqual(result["total_views"], 10)
        self.assertEqual(result["unique_views"], 5)
        self.assertIsNotNone(result["first_viewed_at"])
        self.assertIsNotNone(result["last_viewed_at"])

    def test_get_stats_returns_zero_for_nonexistent(self):
        """Test that get_stats returns zeros for non-existent object."""
        result = self.provider.get_stats(
            f"{self.content_type.app_label}.{self.content_type.model}", 999
        )

        self.assertEqual(result["total_views"], 0)
        self.assertEqual(result["unique_views"], 0)
        self.assertIsNone(result["first_viewed_at"])
        self.assertIsNone(result["last_viewed_at"])

    def test_check_unique_view_returns_true_for_new_session(self):
        """Test that check_unique_view returns True for new session."""
        is_unique = self.provider.check_unique_view(
            "new-session",
            f"{self.content_type.app_label}.{self.content_type.model}",
            1,
        )

        self.assertTrue(is_unique)

    def test_check_unique_view_returns_false_for_existing_session(self):
        """Test that check_unique_view returns False for existing session."""
        # Create an existing event
        PageViewEvent.objects.create(
            content_type=self.content_type,
            object_id=1,
            session_key="existing-session",
            url="/test/",
        )

        is_unique = self.provider.check_unique_view(
            "existing-session",
            f"{self.content_type.app_label}.{self.content_type.model}",
            1,
        )

        self.assertFalse(is_unique)


class AsyncDatabaseProviderTest(TestCase):
    """Test AsyncDatabaseProvider functionality."""

    def setUp(self):
        """Set up test data."""
        self.provider = AsyncDatabaseProvider()
        self.content_type = ContentType.objects.get_for_model(PageViewStatistics)

    def test_async_provider_instance(self):
        """Test that AsyncDatabaseProvider can be instantiated."""
        self.assertIsInstance(self.provider, AsyncDatabaseProvider)
        self.assertIsInstance(self.provider._sync_provider, DatabaseProvider)


class ProviderRegistryTest(TestCase):
    """Test ProviderRegistry functionality."""

    def test_get_provider_returns_sync_by_default(self):
        """Test that get_provider returns sync provider by default."""
        provider = ProviderRegistry.get_provider()
        self.assertIsInstance(provider, DatabaseProvider)

    def test_get_provider_returns_async_when_requested(self):
        """Test that get_provider returns async provider when requested."""
        provider = ProviderRegistry.get_provider(use_async=True)
        self.assertIsInstance(provider, AsyncDatabaseProvider)

    def test_get_async_provider_convenience_method(self):
        """Test get_async_provider convenience method."""
        provider = ProviderRegistry.get_async_provider()
        self.assertIsInstance(provider, AsyncDatabaseProvider)



from abc import ABC, abstractmethod
from typing import Any, Dict


class BaseProvider(ABC):
    """
    Base provider interface for djinsight.
    Implementations can be sync or async depending on use case.
    """

    @abstractmethod
    def record_view(self, event_data: Dict[str, Any]) -> Dict[str, Any]:
        """Record a page view event."""
        pass

    @abstractmethod
    def get_stats(self, content_type: str, object_id: int) -> Dict[str, Any]:
        """Get statistics for an object."""
        pass

    @abstractmethod
    def check_unique_view(
        self, session_key: str, content_type: str, object_id: int
    ) -> bool:
        """Check if this is a unique view for the session."""
        pass

    @abstractmethod
    def increment_counter(self, key: str, amount: int = 1) -> int:
        """Increment a counter (used by Redis provider)."""
        pass

    @abstractmethod
    def mark_viewed(
        self, session_key: str, content_type: str, object_id: int, ttl: int = 86400
    ) -> None:
        """Mark object as viewed by session (used by Redis provider)."""
        pass


class AsyncBaseProvider(ABC):
    """
    Async base provider interface for djinsight.
    Use this for async Django views or when using async database backends.
    """

    @abstractmethod
    async def record_view(self, event_data: Dict[str, Any]) -> Dict[str, Any]:
        """Record a page view event asynchronously."""
        pass

    @abstractmethod
    async def get_stats(self, content_type: str, object_id: int) -> Dict[str, Any]:
        """Get statistics for an object asynchronously."""
        pass

    @abstractmethod
    async def check_unique_view(
        self, session_key: str, content_type: str, object_id: int
    ) -> bool:
        """Check if this is a unique view for the session asynchronously."""
        pass

    @abstractmethod
    async def increment_counter(self, key: str, amount: int = 1) -> int:
        """Increment a counter asynchronously."""
        pass

    @abstractmethod
    async def mark_viewed(
        self, session_key: str, content_type: str, object_id: int, ttl: int = 86400
    ) -> None:
        """Mark object as viewed by session asynchronously."""
        pass



from django.apps import AppConfig


class BlogConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "blog"

    def ready(self):
        try:
            from blog.models import Post
            from djinsight.models import ContentTypeRegistry

            ContentTypeRegistry.register(Post)
        except Exception:
            # Database not ready yet (e.g., during migrations)
            pass



import json
import logging
from typing import Any, Dict, Optional

from django.contrib.contenttypes.models import ContentType
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_http_methods

from djinsight.models import MCPAPIKey, PageViewStatistics, StatsQueryMixin

logger = logging.getLogger(__name__)


class MCPServer:
    """
    Model Context Protocol server for djinsight.
    Exposes Django app statistics to AI agents through MCP.
    """

    @staticmethod
    def get_tools():
        """Return available MCP tools."""
        return [
            {
                "name": "get_page_stats",
                "description": "Get page view statistics for a specific object",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "content_type": {
                            "type": "string",
                            "description": "Content type in format 'app_label.model' (e.g., 'blog.post')",
                        },
                        "object_id": {"type": "number", "description": "Object ID"},
                    },
                    "required": ["content_type", "object_id"],
                },
            },
            {
                "name": "get_top_pages",
                "description": "Get top performing pages by views",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "content_type": {
                            "type": "string",
                            "description": "Content type in format 'app_label.model' (e.g., 'blog.Article')",
                        },
                        "limit": {
                            "type": "number",
                            "description": "Number of results to return (default: 10)",
                        },
                        "metric": {
                            "type": "string",
                            "enum": ["total_views", "unique_views"],
                            "description": "Metric to sort by (default: total_views)",
                        },
                    },
                    "required": ["content_type"],
                },
            },
            {
                "name": "get_period_stats",
                "description": "Get statistics for a specific time period",
                "inputSchema": {
                    "type": "object",
                    "properties": {
                        "content_type": {
                            "type": "string",
                            "description": "Content type in format 'app_label.model'",
                        },
                        "object_id": {"type": "number", "description": "Object ID"},
                        "period": {
                            "type": "string",
                            "enum": ["today", "week", "month", "year"],
                            "description": "Time period",
                        },
                    },
                    "required": ["content_type", "object_id", "period"],
                },
            },
            {
                "name": "list_tracked_models",
                "description": "List all content types that are being tracked",
                "inputSchema": {
                    "type": "object",
                    "properties": {},
                },
            },
        ]

    @staticmethod
    def execute_tool(tool_name: str, arguments: Dict[str, Any]) -> Dict[str, Any]:
        """Execute an MCP tool."""
        try:
            if tool_name == "get_page_stats":
                return MCPServer._get_page_stats(arguments)
            elif tool_name == "get_top_pages":
                return MCPServer._get_top_pages(arguments)
            elif tool_name == "get_period_stats":
                return MCPServer._get_period_stats(arguments)
            elif tool_name == "list_tracked_models":
                return MCPServer._list_tracked_models(arguments)
            else:
                return {"error": f"Unknown tool: {tool_name}"}
        except Exception as e:
            logger.error(f"Error executing tool {tool_name}: {e}")
            return {"error": str(e)}

    @staticmethod
    def _parse_content_type(content_type_str: str) -> Optional[ContentType]:
        """Parse content type string 'app_label.model' into ContentType."""
        try:
            app_label, model = content_type_str.split(".")
            return ContentType.objects.get(app_label=app_label, model=model.lower())
        except (ValueError, ContentType.DoesNotExist) as e:
            logger.error(f"Invalid content type: {content_type_str} - {e}")
            return None

    @staticmethod
    def _get_page_stats(arguments: Dict[str, Any]) -> Dict[str, Any]:
        """Get statistics for a specific page."""
        content_type_str = arguments.get("content_type")
        object_id = arguments.get("object_id")

        ct = MCPServer._parse_content_type(content_type_str)
        if not ct:
            return {"error": f"Invalid content type: {content_type_str}"}

        stats = PageViewStatistics.objects.filter(
            content_type=ct, object_id=object_id
        ).first()

        if not stats:
            return {
                "content_type": content_type_str,
                "object_id": object_id,
                "total_views": 0,
                "unique_views": 0,
                "first_viewed_at": None,
                "last_viewed_at": None,
            }

        return {
            "content_type": content_type_str,
            "object_id": object_id,
            "total_views": stats.total_views,
            "unique_views": stats.unique_views,
            "first_viewed_at": stats.first_viewed_at.isoformat()
            if stats.first_viewed_at
            else None,
            "last_viewed_at": stats.last_viewed_at.isoformat()
            if stats.last_viewed_at
            else None,
        }

    @staticmethod
    def _get_top_pages(arguments: Dict[str, Any]) -> Dict[str, Any]:
        """Get top performing pages."""
        content_type_str = arguments.get("content_type")
        limit = arguments.get("limit", 10)
        metric = arguments.get("metric", "total_views")

        ct = MCPServer._parse_content_type(content_type_str)
        if not ct:
            return {"error": f"Invalid content type: {content_type_str}"}

        stats = PageViewStatistics.objects.filter(content_type=ct).order_by(
            f"-{metric}"
        )[:limit]

        return {
            "content_type": content_type_str,
            "metric": metric,
            "results": [
                {
                    "object_id": s.object_id,
                    "total_views": s.total_views,
                    "unique_views": s.unique_views,
                    "last_viewed_at": s.last_viewed_at.isoformat()
                    if s.last_viewed_at
                    else None,
                }
                for s in stats
            ],
        }

    @staticmethod
    def _get_period_stats(arguments: Dict[str, Any]) -> Dict[str, Any]:
        """Get statistics for a time period."""
        content_type_str = arguments.get("content_type")
        object_id = arguments.get("object_id")
        period = arguments.get("period", "week")

        ct = MCPServer._parse_content_type(content_type_str)
        if not ct:
            return {"error": f"Invalid content type: {content_type_str}"}

        try:
            model_class = ct.model_class()
            obj = model_class.objects.get(pk=object_id)
        except Exception as e:
            return {"error": f"Object not found: {e}"}

        period_map = {
            "today": StatsQueryMixin.get_views_today,
            "week": StatsQueryMixin.get_views_week,
            "month": StatsQueryMixin.get_views_month,
            "year": StatsQueryMixin.get_views_year,
        }

        handler = period_map.get(period)
        if not handler:
            return {"error": f"Invalid period: {period}"}

        views = handler(obj, chart_data=True)

        return {
            "content_type": content_type_str,
            "object_id": object_id,
            "period": period,
            "data": views,
        }

    @staticmethod
    def _list_tracked_models(arguments: Dict[str, Any]) -> Dict[str, Any]:
        """List all tracked content types."""
        from djinsight.models import ContentTypeRegistry

        registries = ContentTypeRegistry.objects.filter(enabled=True).select_related(
            "content_type"
        )

        return {
            "tracked_models": [
                {
                    "app_label": r.content_type.app_label,
                    "model": r.content_type.model,
                    "content_type": f"{r.content_type.app_label}.{r.content_type.model}",
                    "track_anonymous": r.track_anonymous,
                    "track_authenticated": r.track_authenticated,
                }
                for r in registries
            ]
        }


@csrf_exempt
@require_http_methods(["POST"])
def mcp_endpoint(request):
    """Main MCP endpoint."""
    try:
        auth_header = request.headers.get("Authorization")
        if not auth_header or not auth_header.startswith("Bearer "):
            return JsonResponse(
                {"error": "Missing or invalid Authorization header"}, status=401
            )

        api_key = auth_header[7:]
        if not MCPAPIKey.validate_key(api_key):
            return JsonResponse({"error": "Invalid API key"}, status=403)

        data = json.loads(request.body)
        action = data.get("action")

        if action == "list_tools":
            tools = MCPServer.get_tools()
            return JsonResponse({"tools": tools})

        elif action == "execute_tool":
            tool_name = data.get("tool_name")
            arguments = data.get("arguments", {})

            result = MCPServer.execute_tool(tool_name, arguments)
            return JsonResponse(result)

        else:
            return JsonResponse({"error": "Invalid action"}, status=400)

    except json.JSONDecodeError:
        return JsonResponse({"error": "Invalid JSON"}, status=400)
    except Exception as e:
        logger.error(f"MCP endpoint error: {e}")
        return JsonResponse({"error": "Internal server error"}, status=500)



# Generated by Django 6.0 on 2025-12-09 12:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("djinsight", "0004_v2_new_architecture"),
    ]

    operations = [
        migrations.CreateModel(
            name="MCPAPIKey",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "key",
                    models.CharField(
                        max_length=64, unique=True, verbose_name="API Key"
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="Name")),
                (
                    "description",
                    models.TextField(blank=True, verbose_name="Description"),
                ),
                ("is_active", models.BooleanField(default=True, verbose_name="Active")),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "last_used_at",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="Last Used At"
                    ),
                ),
            ],
            options={
                "verbose_name": "MCP API Key",
                "verbose_name_plural": "MCP API Keys",
            },
        ),
        migrations.RemoveIndex(
            model_name="pageviewlog",
            name="djinsight_p_page_id_f86134_idx",
        ),
        migrations.RemoveIndex(
            model_name="pageviewlog",
            name="djinsight_p_content_4f4405_idx",
        ),
        migrations.RemoveIndex(
            model_name="pageviewlog",
            name="djinsight_p_session_54f5b5_idx",
        ),
        migrations.RemoveIndex(
            model_name="pageviewlog",
            name="djinsight_p_timesta_9736c6_idx",
        ),
        migrations.DeleteModel(
            name="PageViewLog",
        ),
        migrations.AlterModelOptions(
            name="pageviewstatistics",
            options={
                "verbose_name": "Page View Statistics",
                "verbose_name_plural": "Page View Statistics",
            },
        ),
        migrations.RemoveIndex(
            model_name="pageviewsummary",
            name="djinsight_p_page_id_efa5ff_idx",
        ),
        migrations.RenameIndex(
            model_name="contenttyperegistry",
            new_name="djinsight_c_content_6272ec_idx",
            old_name="djinsight_c_content_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewevent",
            new_name="djinsight_p_content_2dc625_idx",
            old_name="djinsight_p_content_obj_tim_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewevent",
            new_name="djinsight_p_session_8f6dd6_idx",
            old_name="djinsight_p_session_con_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewevent",
            new_name="djinsight_p_timesta_684154_idx",
            old_name="djinsight_p_timesta_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewevent",
            new_name="djinsight_p_content_bb1ec0_idx",
            old_name="djinsight_p_content_tim_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewstatistics",
            new_name="djinsight_p_content_5a1d84_idx",
            old_name="djinsight_p_content_obj_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewstatistics",
            new_name="djinsight_p_content_8a4fc3_idx",
            old_name="djinsight_p_content_tot_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewstatistics",
            new_name="djinsight_p_content_943b3e_idx",
            old_name="djinsight_p_content_uni_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewstatistics",
            new_name="djinsight_p_last_vi_195672_idx",
            old_name="djinsight_p_last_vi_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewstatistics",
            new_name="djinsight_p_updated_d991b2_idx",
            old_name="djinsight_p_updated_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewsummary",
            new_name="djinsight_p_content_8d9111_idx",
            old_name="djinsight_p_content_obj_dat_idx",
        ),
        migrations.RenameIndex(
            model_name="pageviewsummary",
            new_name="djinsight_p_content_cc7390_idx",
            old_name="djinsight_p_content_912dc8_idx",
        ),
        migrations.AlterField(
            model_name="pageviewsummary",
            name="date",
            field=models.DateField(db_index=True, verbose_name="Date"),
        ),
        migrations.AlterField(
            model_name="pageviewsummary",
            name="object_id",
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterField(
            model_name="pageviewsummary",
            name="total_views",
            field=models.PositiveIntegerField(default=0, verbose_name="Total Views"),
        ),
        migrations.AlterField(
            model_name="pageviewsummary",
            name="unique_views",
            field=models.PositiveIntegerField(default=0, verbose_name="Unique Views"),
        ),
        migrations.AddIndex(
            model_name="mcpapikey",
            index=models.Index(
                fields=["key", "is_active"], name="djinsight_m_key_74ae77_idx"
            ),
        ),
    ]
